// A Java version for the Kaser Interpreter - The Component Listener
// For some of my own fun!
// Stephen M. Bennett
// 29Jun02 Started

package stephen.bennett.kint;

import java.awt.event.*;


// Map interesting container events to calls on KE Event

class KComponentListener extends ComponentAdapter {
  // the component (window) has been moved, call KE Event
  public void componentMoved(ComponentEvent e) {
    Log.fine("Component Moved event happened");

    // if we are to ignore this move, ignore it (but not subsequent ones)
    if (UI.getIgnoreNextWindowMove()) {
      Log.fine("Ignoring this move event");
      UI.setIgnoreNextWindowMove(false);
    }
    else {
      // find the new position
      int x = UI.getX();
      int y = UI.getY();
      Log.fine("New location", x, y);

      // call the .KE WINDOW_MOVE function
      Log.info("Calling KEEVENT_WINDOW_MOVE");
      KInt.keEvent(KInt.KEEVENT_WINDOW_MOVE, x, y, 0);
    }

    // this was an interesting event
    UI.setMostRecentEventTime();
  }


  // the component (window) has been resized, call KE Event
  public void componentResized(ComponentEvent e) {
    Log.fine("Component Resize event happened");

    // if we are to ignore this resize, ignore it (but not subsequent ones)
    if (UI.getIgnoreNextWindowSize()) {
      Log.fine("Ignoring this move resize");
      UI.setIgnoreNextWindowSize(false);
    }
    else {
      // find the width and height position
      int w = UI.getWidth();
      int h = UI.getHeight();
      Log.fine("New size", w, h);

      // call the .KE WINDOW_SIZE function
      Log.info("Calling KEEVENT_WINDOW_SIZE");
      KInt.keEvent(KInt.KEEVENT_WINDOW_SIZE, w, h, 0);
    }

    // this was an interesting event
    UI.setMostRecentEventTime();
  }



  // Return our CVS id
  public static String getCVSID() {
    return "$Id: KComponentListener.java,v 1.3 2002/10/06 15:28:54 Bennett Stephen Exp $";
  }

}


/*
 * $Log: KComponentListener.java,v $
 * Revision 1.3  2002/10/06 15:28:54  Bennett Stephen
 * CVS method added
 *
 * Revision 1.2  2002/09/08 19:44:14  Bennett Stephen
 * code for ignoring events
 *
 * Revision 1.1  2002/08/07 19:22:22  Bennett Stephen
 * Initial Version
 *
 *
 */

// ----------------------------------------------------
// Emacs magic - please do not remove the following lines
/*
 * Local Variables:
 * write-contents-hooks: ((lambda () (untabify (point-min) (point-max))))
 * End:
*/
// ----------------------------------------------------
