// A Java version for the Kaser Interpreter - The Key Listener
// For some of my own fun!
// Stephen M. Bennett
// 29Jun02 Started

package stephen.bennett.kint;


// Map interesting key events to calls on KE Event

import java.awt.event.*;


class KKeyListener extends KeyAdapter {
  // Key codes as understood by KE programs
  public static final int K_LBUTT     = 0x01;
  public static final int K_RBUTT     = 0x02;
  public static final int K_MBUTT     = 0x04;

  public static final int K_BACK      = 0x08;
  public static final int K_TAB       = 0x09;
  public static final int K_ENTER     = 0x0D;

  public static final int K_SHIFT     = 0x10;
  public static final int K_CTL       = 0x11;
  public static final int K_MENU      = 0x12;
  public static final int K_ESC       = 0x1B;

  public static final int K_SPACE     = 0x20;
  public static final int K_PGUP      = 0x21;
  public static final int K_PGDN      = 0x22;
  public static final int K_END       = 0x23;
  public static final int K_HOME      = 0x24;
  public static final int K_LFCURS    = 0x25;
  public static final int K_UPCURS    = 0x26;
  public static final int K_RTCURS    = 0x27;
  public static final int K_DNCURS    = 0x28;
  public static final int K_INS       = 0x2D;
  public static final int K_DEL       = 0x2E;

  public static final int K_0         = 0x30;
  public static final int K_1         = 0x31;
  public static final int K_2         = 0x32;
  public static final int K_3         = 0x33;
  public static final int K_4         = 0x34;
  public static final int K_5         = 0x35;
  public static final int K_6         = 0x36;
  public static final int K_7         = 0x37;
  public static final int K_8         = 0x38;
  public static final int K_9         = 0x39;

  public static final int K_A         = 0x41;
  public static final int K_B         = 0x42;
  public static final int K_C         = 0x43;
  public static final int K_D         = 0x44;
  public static final int K_E         = 0x45;
  public static final int K_F         = 0x46;
  public static final int K_G         = 0x47;
  public static final int K_H         = 0x48;
  public static final int K_I         = 0x49;
  public static final int K_J         = 0x4A;
  public static final int K_K         = 0x4B;
  public static final int K_L         = 0x4C;
  public static final int K_M         = 0x4D;
  public static final int K_N         = 0x4E;
  public static final int K_O         = 0x4F;
  public static final int K_P         = 0x50;
  public static final int K_Q         = 0x51;
  public static final int K_R         = 0x52;
  public static final int K_S         = 0x53;
  public static final int K_T         = 0x54;
  public static final int K_U         = 0x55;
  public static final int K_V         = 0x56;
  public static final int K_W         = 0x57;
  public static final int K_X         = 0x58;
  public static final int K_Y         = 0x59;
  public static final int K_Z         = 0x5A;

  public static final int K_NUMPAD0   = 0x60;
  public static final int K_NUMPAD1   = 0x61;
  public static final int K_NUMPAD2   = 0x62;
  public static final int K_NUMPAD3   = 0x63;
  public static final int K_NUMPAD4   = 0x64;
  public static final int K_NUMPAD5   = 0x65;
  public static final int K_NUMPAD6   = 0x66;
  public static final int K_NUMPAD7   = 0x67;
  public static final int K_NUMPAD8   = 0x68;
  public static final int K_NUMPAD9   = 0x69;
  public static final int K_MULTIPLY  = 0x6A;
  public static final int K_NUMPADADD = 0x6B;
  public static final int K_SEPARATOR = 0x6C;
  public static final int K_NUMPADSUB = 0x6D;
  public static final int K_DECIMAL   = 0x6E;
  public static final int K_NUMPADDIV = 0x6F;
  public static final int K_F1        = 0x70;
  public static final int K_F2        = 0x71;
  public static final int K_F3        = 0x72;
  public static final int K_F4        = 0x73;
  public static final int K_F5        = 0x74;
  public static final int K_F6        = 0x75;
  public static final int K_F7        = 0x76;
  public static final int K_F8        = 0x77;
  public static final int K_F9        = 0x78;
  public static final int K_F10       = 0x79;
  public static final int K_F11       = 0x7A;
  public static final int K_F12       = 0x7B;

  public static final int K_NUMLOCK   = 0x90;
  public static final int K_SCROLL    = 0x91;

  public static final int K_EQPLUS    = 0xBB;    // =+ key
  public static final int K_UNDERSUB  = 0xBD;    // -_ key
  public static final int K_SLASH     = 0xBF;    // /? key

  public static final int K_BACKBUTT  = 0x8001;  // mouse BACK button
  public static final int K_FWDBUTT   = 0x8002;  // mouse FWD button


  // for the debug when a key is pressed
  private static boolean debugOnKey = false;


  // a key has been typed, call KE Event
  public void keyTyped(KeyEvent e) {
    Log.fine("Key Typed event happened");
    // System.out.println("keyTyped");

    // convert Java char to KE value
    int k = keKeyValue(e.getKeyChar());

    // switch on debugging corresponding key pressed
    if (debugOnKey) {
      if (k == K_F) {
        Log.setLevelFine();
      }
      else if (k == K_I) {
        Log.setLevelInfo();
      }
    }

    // using the same approach as MAIN.CPP, only send if char is " " or higher
    if (k >= 0x20) {
      // call the .KE function
      Log.info("Calling KEEVENT_CHAR");
      // we always get one char at a time
      KInt.keEvent(KInt.KEEVENT_CHAR, k, 1, 0);

      // this was an interesting event
      UI.setMostRecentEventTime();
    }
  }


  // a key has been pressed, call KE Event
  public void keyPressed(KeyEvent e) {
    Log.fine("Key Pressed event happened");
    // System.out.println("keyPressed");

    // convert Java code to KE value
    int k = keKeyValue(e.getKeyCode());

    // using the same approach as MAIN.CPP, 0-9A-Z go to ALPHAKEY_DOWN
    // rest to KEY_DOWN
    if ((k >= 0x30 && k <= 0x39) || (k >= 0x41 && k <= 0x5A)) {
      // call the .KE function
      Log.info("Calling KEEVENT_ALPHAKEY_DOWN");
      // we always get one char at a time
      KInt.keEvent(KInt.KEEVENT_ALPHAKEY_DOWN, k, 1, 0);
    }
    else {
      // call the .KE function
      Log.info("Calling KEEVENT_KEY_DOWN");
      // we always get one char at a time
      KInt.keEvent(KInt.KEEVENT_KEY_DOWN, k, 1, 0);
    }

    // this was an interesting event
    UI.setMostRecentEventTime();
  }


  // a key has been released, call KE Event
  public void keyReleased(KeyEvent e) {
    Log.fine("Key Released event happened");
    // System.out.println("keyReleased");

    // convert Java code to KE value
    int k = keKeyValue(e.getKeyCode());

    // using the same approach as MAIN.CPP, 0-9A-Z go to ALPHAKEY_UP
    // rest to KEY_UP
    if ((k >= 0x30 && k <= 0x39) || (k >= 0x41 && k <= 0x5A)) {
      // call the .KE function
      Log.info("Calling KEEVENT_ALPHAKEY_UP");
      // we always get one char at a time
      KInt.keEvent(KInt.KEEVENT_ALPHAKEY_UP, k, 1, 0);
    }
    else {
      // call the .KE function
      Log.info("Calling KEEVENT_KEY_UP");
      // we always get one char at a time
      KInt.keEvent(KInt.KEEVENT_KEY_UP, k, 1, 0);
    }

    // this was an interesting event
    UI.setMostRecentEventTime();
  }




  // common method to convert Java key code to KE code value
  private static int keKeyValue(int code) {
    Log.fine("key code", code);
    // System.out.println("java code "+Integer.toHexString(code));

    // convert codes
    switch(code) {

    case KeyEvent.VK_ENTER:
      return K_ENTER;

    case KeyEvent.VK_INSERT:
      return K_INS;

    case KeyEvent.VK_DELETE:
      return K_DEL;

    default:
      // no conversion needed
      return code;
    }
  }


  // notes that we will start debugging when the right key is pressed
  public static void setDebugOnKey() {
    debugOnKey = true;
  }


  // Return our CVS id
  public static String getCVSID() {
    return "$Id: KKeyListener.java,v 1.5 2003/01/21 15:47:21 Bennett Stephen Exp $";
  }


}


/*
 * $Log: KKeyListener.java,v $
 * Revision 1.5  2003/01/21 15:47:21  Bennett Stephen
 * support for debug on key flag
 *
 * Revision 1.4  2002/10/06 15:28:41  Bennett Stephen
 * CVS method added
 *
 * Revision 1.3  2002/09/08 19:44:36  Bennett Stephen
 * commented out debugging
 *
 * Revision 1.2  2002/08/10 08:23:55  Bennett Stephen
 * java key codes mapped to windows
 *
 * Revision 1.1  2002/08/07 19:22:21  Bennett Stephen
 * Initial Version
 *
 *
 */

// ----------------------------------------------------
// Emacs magic - please do not remove the following lines
/*
 * Local Variables:
 * write-contents-hooks: ((lambda () (untabify (point-min) (point-max))))
 * End:
*/
// ----------------------------------------------------
