// A Java version for the Kaser Interpreter - The Mouse Listener
// For some of my own fun!
// Stephen M. Bennett
// 29Jun02 Started

package stephen.bennett.kint;

import java.awt.event.*;


// Map interesting mouse events to calls on KE Event

class KMouseListener extends MouseAdapter {

  // the mouse is pressed
  public void mousePressed(MouseEvent e) {
    Log.fine("Mouse Pressed event happened");
    // Log.fine("MouseEvent is "+e);

    int flags = mouseEventToFlags(e);
    int coords = mouseEventToXYD(e);
    int mouseKeyCode = mouseEventToMouseKeyCode(e);

    // call the .KE function
    Log.info("Calling KEEVENT_KEY_DOWN");
    KInt.keEvent(KInt.KEEVENT_KEY_DOWN, mouseKeyCode, flags, coords);

    // this was an interesting event
    UI.setMostRecentEventTime();
  }



  // the mouse is released
  public void mouseReleased(MouseEvent e) {
    Log.fine("Mouse Released event happened");
    // Log.fine("MouseEvent is "+e);

    int flags = mouseEventToFlags(e);
    int coords = mouseEventToXYD(e);
    int mouseKeyCode = mouseEventToMouseKeyCode(e);

    // call the .KE function
    Log.info("Calling KEEVENT_KEY_UP");
    KInt.keEvent(KInt.KEEVENT_KEY_UP, mouseKeyCode, flags, coords);

    // this was an interesting event
    UI.setMostRecentEventTime();
  }



  // common method convert a MouseEvent x and y to DWORD (LOWORD X)
  public static int mouseEventToXYD(MouseEvent e) {
    int x = e.getX();
    int y = e.getY();
    int result = (y << 16)|x;    
    Log.fine("XY as DWORD", result);
    return result;
  }



  // common method convert a MouseEvent modifiers into flags required by .KE
  public static int mouseEventToFlags(MouseEvent e) {
    // decide which buttons are down
    int mods = e.getModifiersEx();
    boolean leftButtonDown = ((mods & InputEvent.BUTTON1_DOWN_MASK) != 0);
    boolean rightButtonDown = ((mods & InputEvent.BUTTON3_DOWN_MASK) != 0);
    boolean middleButtonDown = ((mods & InputEvent.BUTTON2_DOWN_MASK) != 0);
    boolean shiftKeyDown = ((mods & InputEvent.SHIFT_DOWN_MASK) != 0);
    boolean ctrlKeyDown = ((mods & InputEvent.CTRL_DOWN_MASK) != 0);

    // set bits according to .KE needs
    int flags = 0;
    if (leftButtonDown) flags |= 0x01;
    if (rightButtonDown) flags |= 0x02;
    if (shiftKeyDown) flags |= 0x04;
    if (ctrlKeyDown) flags |= 0x08;
    if (middleButtonDown) flags |= 0x10;
    Log.fine("flags", flags);

    return flags;
  }


  // common method to convert a MouseEvent KE mouse key code
  private static int mouseEventToMouseKeyCode(MouseEvent e) {
    // left button
    if (e.getButton() == MouseEvent.BUTTON1) {
      return KKeyListener.K_LBUTT;
    }

    // right button
    if (e.getButton() == MouseEvent.BUTTON3) {
      return KKeyListener.K_RBUTT;
    }

    // middle button
    if (e.getButton() == MouseEvent.BUTTON2) {
      return KKeyListener.K_MBUTT;
    }

    Log.errorExit("Cannot convert MouseEvent to MouseKeyCode: "+e);
    return 0;
  }




  // Return our CVS id
  public static String getCVSID() {
    return "$Id: KMouseListener.java,v 1.2 2002/10/06 15:28:30 Bennett Stephen Exp $";
  }

}


/*
 * $Log: KMouseListener.java,v $
 * Revision 1.2  2002/10/06 15:28:30  Bennett Stephen
 * CVS method added
 *
 * Revision 1.1  2002/08/07 19:22:21  Bennett Stephen
 * Initial Version
 *
 *
 */

// ----------------------------------------------------
// Emacs magic - please do not remove the following lines
/*
 * Local Variables:
 * write-contents-hooks: ((lambda () (untabify (point-min) (point-max))))
 * End:
*/
// ----------------------------------------------------
