// A Java version for the Kaser Interpreter - The JPanel
// For some of my own fun!
// Stephen M. Bennett
// 29Jun02 Started

package stephen.bennett.kint;

import java.awt.*;
import javax.swing.*;


// The main paint area of the program is a JPanel (i.e. using Swing)
// Here we can paint the kBM (Image) on to the screen and translate
// key events, etc. to calls on the .KE

// the KASM code does ALL of it's drawing to kBM, a memory bitmap.
// when Windows needs to update the REAL screen, the WM_PAINT message
// in this program merely BitBlt's from kBM to the screen DC.
// - for Java, we hold an Image that all painting is done to and the
// - paintComponent draws that Image onto the screen

class KPanel extends JPanel {
  // the Image that all draw events go to
  private Image _image;

  static int count;

  // get, and create if not done already, our image
  public Image getImage() {
    // if not image yet, create and return it
    if (_image == null) {
      int w = getWidth();
      int h = getHeight();
      Log.fine("no image yet, creating new one w h", w, h);
      _image = createImage(w, h);
      // Log.fine("created Image", _image);
      if (_image == null) {
        Log.errorExit("Could not create image");
      }
      return _image;
    }

    // check the width and height is still the same as we are
    // if not, recreate the image
    if (_image.getWidth(null) != getWidth() ||
        _image.getHeight(null) != getHeight()) {
      
      // need to create a new one
      int w = getWidth();
      int h = getHeight();
      Log.fine("Our width and/or height differs from our image, recreating w h",
               w, h);
      Image newimage = createImage(w, h);
      // Log.fine("created Image", newimage);
      if (newimage == null) {
        Log.errorExit("Could not create image");
      }

      // draw the old image onto this one
      newimage.getGraphics().drawImage(_image, 0, 0, null);

      _image = newimage;
    }

    return _image;
  }


  // to paint my component just overlay my image
  public void paintComponent(Graphics g) {
    // count++;
    // System.out.println("paintComponent "+count);
    
    Log.info("KPanel paintComponent");
    // System.out.println("KPanel paintComponent");

    // get our JPanel to paint first
    super.paintComponent(g);

    // draw our image, if it has been defined
    Image image = getImage();
    if (image != null) {
      g.drawImage(image, 0, 0, null);
    }
  }



  // Return our CVS id
  public static String getCVSID() {
    return "$Id: KPanel.java,v 1.4 2002/11/28 19:32:30 Bennett Stephen Exp $";
  }


}


/*
 * $Log: KPanel.java,v $
 * Revision 1.4  2002/11/28 19:32:30  Bennett Stephen
 * release candidate 1 version
 *
 * Revision 1.3  2002/10/06 15:27:44  Bennett Stephen
 * CVS method added
 *
 * Revision 1.2  2002/09/08 19:45:16  Bennett Stephen
 * logging and comment changes
 *
 * Revision 1.1  2002/08/07 19:22:21  Bennett Stephen
 * Initial Version
 *
 *
 */

// ----------------------------------------------------
// Emacs magic - please do not remove the following lines
/*
 * Local Variables:
 * write-contents-hooks: ((lambda () (untabify (point-min) (point-max))))
 * End:
*/
// ----------------------------------------------------
