// A Java version for the Kaser Interpreter - some test routines
// For some of my own fun!
// Stephen M. Bennett
// 29Jun02 Started


package stephen.bennett.kint;


public class KTests {
  // a copy of  main registers for testing
  // 32*4 + 48*2 + 32*1 = 256 (32 DWORDS, 48 WORDS, 32 BYTES)
  private static byte[] myRegs = new byte[256]; 



  // some testing
  public static void runTests(String testName) {
    if (testName.equals("SIGNEDBYTEDEC")) signedByteDec();
    if (testName.equals("SIGNEDBYTEADD")) signedByteAdd();
    if (testName.equals("SIGNEDBYTEMUL")) signedByteMul();
    if (testName.equals("SIGNEDBYTEDIV")) signedByteDiv();
    if (testName.equals("OTHERS")) otherTests();
  }


  // some other tests
  private static void otherTests() {
    System.out.println("1f8 is "+(0x1f8));
    System.out.println("fffffff8 is "+(0xfffffff8));
    System.out.println("");
    byte[] kode = new byte[2];
    kode[0] = (byte)0xfe;
    kode[1] = (byte)0xff;
    int opcode = Mem.GETMEMW(kode, 0) & 0xffff;
    int reg = opcode & 0xff;
    System.out.println("reg is "+reg);
  }


  // signed and unsigned byte dec
  private static void signedByteDec() {
    byte b = -128;
    System.out.println("doing "+b+" - 1");
    short wVal = Mem.bUnsigned(b);
    System.out.println("as unsigned "+wVal);
    byte b1 = (byte)((byte)wVal - 1);
    System.out.println("signed dec "+b1);
    byte b2 = (byte)(Mem.bUnsigned((byte)wVal) - 1);
    System.out.println("unsigned dec byte "+b2);
    short s2 = (short)(Mem.bUnsigned((byte)wVal) - 1);
    System.out.println("unsigned dec short "+s2);
  }

  // signed and unsigned byte addition
  private static void signedByteMul() {
    byte b = -1;
    System.out.println("doing 1 * "+b);
    short wVal = (short)b;
    Mem.SETREGB(myRegs, Mem.REG_80, (byte)0x01);
    byte b1 = (byte)(Mem.GETREGB(myRegs, Mem.REG_80) * (byte)wVal);
    System.out.println("signed mul "+b1);
    byte b2 = (byte)(Mem.bUnsigned(Mem.GETREGB(myRegs, Mem.REG_80)) *
                     Mem.bUnsigned((byte)wVal));
    System.out.println("unsigned mul byte "+b2);
    short s2 = (short)(Mem.bUnsigned(Mem.GETREGB(myRegs, Mem.REG_80)) *
                       Mem.bUnsigned((byte)wVal));
    System.out.println("unsigned mul short "+s2);
  }

  // signed and unsigned byte add
  private static void signedByteAdd() {
    byte b = -1;
    System.out.println("doing "+b+" + 1");
    short wVal = (short)b;
    Mem.SETREGB(myRegs, Mem.REG_80, (byte)0x01);
    byte b1 = (byte)(Mem.GETREGB(myRegs, Mem.REG_80) + (byte)wVal);
    System.out.println("signed add "+b1);
    byte b2 = (byte)(Mem.bUnsigned(Mem.GETREGB(myRegs, Mem.REG_80)) + 
                     Mem.bUnsigned((byte)wVal));
    System.out.println("unsigned add byte "+b2);
    short s2 = (short)(Mem.bUnsigned(Mem.GETREGB(myRegs, Mem.REG_80)) + 
                       Mem.bUnsigned((byte)wVal));
    System.out.println("unsigned add short "+s2);
  }

  // signed and unsigned byte div
  private static void signedByteDiv() {
    byte b = -1;
    System.out.println("doing 2 / "+b);
    short wVal = b;
    Mem.SETREGB(myRegs, Mem.REG_80, (byte)0x02);
    byte b1 = (byte)(Mem.GETREGB(myRegs, Mem.REG_80) / (byte)wVal);
    System.out.println("signed div "+b1);
    byte b2 = (byte)(Mem.bUnsigned(Mem.GETREGB(myRegs, Mem.REG_80)) / 
                     Mem.bUnsigned((byte)wVal));
    System.out.println("unsigned div byte "+b2);
    short s2 = (short)(Mem.bUnsigned(Mem.GETREGB(myRegs, Mem.REG_80)) / 
                       Mem.bUnsigned((byte)wVal));
    System.out.println("unsigned div short "+s2);
  }



  private static void rest() {
    // find a way to get byte to short and int taking original byte as unsigned
    if (false) {
      System.out.println("byte to short and int test");
      byte b = (byte)0x80;
      System.out.println("byte is "+b);
      short s = (short)(b & 0xff);
      System.out.println("short is "+s);
      int i = b & 0xff;
      System.out.println("int is "+i);
    }

    // find a way to get short to int taking original short as unsigned
    if (false) {
      System.out.println("short to int test");
      short s = (short)0x8000;
      System.out.println("short is "+s);
      int i = s & 0xffff;
      System.out.println("int is "+i);
    }

    // unsigned int must be held as a long
    if (false) {
      System.out.println("int to long test");
      int i = 0x80000000;
      System.out.println("int is "+i);
      long l = i & 0xffffffffL;
      System.out.println("long is "+l);
    }

    if (false) {
      Mem.SETREGB(myRegs, Mem.REG_80, (byte)0x80);
      byte b = Mem.GETREGB(myRegs, Mem.REG_80);
      System.out.println("byte = "+b);
      short s = b;
      System.out.println("as short = "+s);
      int i = b;
      System.out.println("as int = "+i);

      // this shows that the widening keeps the sign (as Java Spec says)
    }

  }


  // Return our CVS id
  public static String getCVSID() {
    return "$Id: KTests.java,v 1.2 2002/10/06 15:27:31 Bennett Stephen Exp $";
  }


}


/*
 * $Log: KTests.java,v $
 * Revision 1.2  2002/10/06 15:27:31  Bennett Stephen
 * CVS method added
 *
 * Revision 1.1  2002/08/07 19:22:21  Bennett Stephen
 * Initial Version
 *
 *
 */

// ----------------------------------------------------
// Emacs magic - please do not remove the following lines
/*
 * Local Variables:
 * write-contents-hooks: ((lambda () (untabify (point-min) (point-max))))
 * End:
*/
// ----------------------------------------------------
