// A Java version for the Kaser Interpreter - some logging methods
// For some of my own fun!
// Stephen M. Bennett
// 29Jun02 Started

package stephen.bennett.kint;


// We use 
//   SEVERE - errors
//   INFO   - KEVENTs
//   FINE   - detail

import java.io.*;


public class Log {
  // here are the levels
  private static final int SEVERE = 1;
  private static final int INFO   = 2;
  private static final int FINE = 3;

  // our level
  private static int level = SEVERE;

  // the print stream we write to
  private static PrintStream logPS = System.out;

  // the date format for SysTimeStr
  // private static java.text.SimpleDateFormat sdf = new java.text.SimpleDateFormat("HH:mm:ss ");



  // set the log level to info
  public static void setLevelInfo() {
    level = INFO;
  }

  // set the log level to fine
  public static void setLevelFine() {
    level = FINE;
  }


  // open the log
  public static void openLog(String filename) {
    try {
      logPS = new PrintStream(new FileOutputStream(filename));
    }
    catch (FileNotFoundException e) {
      System.out.println("failed to open log "+filename);
      logPS = System.out;
    }
  }




  // log a severe message
  public static void severe(String msg) {
    log(SEVERE, msg);
  }

  // log a severe message, with String
  public static void severe(String msg, String s) {
    log(SEVERE, msg, s);
  }

  // log a severe message, with int
  public static void severe(String msg, int i) {
    log(SEVERE, msg, i);
  }

  // log a severe message, with 2 ints
  public static void severe(String msg, int i1, int i2) {
    log(SEVERE, msg, i1, i2);
  }



  // log an info message (if level allows)
  public static void info(String msg) {
    if (level > SEVERE) {
      log(INFO, msg);
    }
  }

  // log an info message, with String (if level allows)
  public static void info(String msg, String s) {
    if (level > SEVERE) {
      log(INFO, msg, s);
    }
  }

  // log an info message, with int (if level allows)
  public static void info(String msg, int i) {
    if (level > SEVERE) {
      log(INFO, msg, i);
    }
  }

  // log an info message, with 2 ints (if level allows)
  public static void info(String msg, int i1, int i2) {
    if (level > SEVERE) {
      log(INFO, msg, i1, i2);
    }
  }



  // log a fine message (if level allows)
  public static void fine(String msg) {
    if (level > INFO) {
      log(FINE, msg);
    }
  }

  // log a fine message, with string (if level allows)
  public static void fine(String msg, String s) {
    if (level > INFO) {
      log(FINE, msg, s);
    }
  }

  // log a fine message, with boolean (if level allows)
  public static void fine(String msg, boolean b) {
    if (level > INFO) {
      log(FINE, msg, b);
    }
  }

  // log a fine message, with byte (if level allows)
  public static void fine(String msg, byte b) {
    if (level > INFO) {
      log(FINE, msg, b);
    }
  }
  
  // log a fine message, with short (if level allows)
  public static void fine(String msg, short s) {
    if (level > INFO) {
      log(FINE, msg, s);
    }
  }
  
  // log a fine message, with int (if level allows)
  public static void fine(String msg, int i) {
    if (level > INFO) {
      log(FINE, msg, i);
    }
  }
  
  // log a fine message, with long (if level allows)
  public static void fine(String msg, long l) {
    if (level > INFO) {
      log(FINE, msg, l);
    }
  }

  // log a fine message, with 2 ints (if level allows)
  public static void fine(String msg, int i1, int i2) {
    if (level > INFO) {
      log(FINE, msg, i1, i2);
    }
  }
  
  // log a fine message, with string and int (if level allows)
  public static void fine(String msg, String msg2, int i) {
    if (level > INFO) {
      log(FINE, msg, msg2, i);
    }
  }
  
  // log a fine message, with 2 longs (if level allows)
  public static void fine(String msg, long l1, long l2) {
    if (level > INFO) {
      log(FINE, msg, l1, l2);
    }
  }
  
  

  // log a DWORD 
  public static void DWORD(String msg, byte[] bytes, int offset) {
    if (level > INFO) {
      fine(msg, Mem.GETMEMD(bytes, offset));
    }
  }

  // log a WORD
  public static void WORD(String msg, byte[] bytes, int offset) {
    if (level > INFO) {
      fine(msg, Mem.GETMEMW(bytes, offset));
    }
  }

  // log a BYTE
  public static void BYTE(String msg, byte[] bytes, int offset) {
    if (level > INFO) {
      fine(msg, Mem.GETMEMB(bytes, offset));
    }
  }




  // log, print error and exit with string
  public static void errorExit(String error, String s) {
    errorExit(error+" : "+s);
  }

  // log, print error and exit with int
  public static void errorExit(String error, int i) {
    errorExit(error+" : "+Integer.toHexString(i));
  }

  // log, print error and exit
  public static void errorExit(String error) {
    errorExit(error, (Exception)null);
  }

  // log, print error and exit, with an exception
  public static void errorExit(String error, Exception e) {
    // log and print the error
    String msg1 = "Error: "+error;
    severe(msg1);
    System.out.println(msg1);

    // log and print the error if present
    if (e != null) {
      String msg2 = "Exception: "+e;
      severe(msg2);
      e.printStackTrace(logPS);
      System.out.println(msg2);
      System.out.println("See log file for more details");
    }
    
    // KaserInterpreter.printMainRegisters();
    // KInt.printPerfInfo();
    System.exit(-1);
  }





  // log the given registers (that are not 0)
  public static void registers(byte[] regs) {
    // we only do this at info level or above
    if (level < INFO) {
      return;
    }

    info("Printing Registers");
    for(int i=0; i<32; i++) {
      int d = Mem.GETMEMD(regs, i*4);
      if (d != 0) {
        info("  r"+i, d);
      }
    }
    for(int i=32; i<80; i++) {
      short s = Mem.GETMEMW(regs, 0x80+((i-32)*2));
      if (s != 0) {
        info("  r"+i, s);
      }
    }
    for(int i=80; i<112; i++) {
      byte b = Mem.GETMEMB(regs, 0xE0+i-80); 
      if (b != 0) {
        info("  r"+i, b);
      }
    }
  }



  // PRIVATE LOGGING METHODS TO DO REAL WORK


  // log the message (regardless of level)
  private static void log(int l, String msg) {
    logTimeAndLevel(l);
    logPS.println(msg);
  }

  // log the message and string (regardless of level)
  private static void log(int l, String msg, String s) {
    logTimeAndLevel(l);
    logPS.print(msg);
    logPS.print(" : ");
    logPS.println(s);
  }

  // log the message and int (regardless of level)
  private static void log(int l, String msg, int i) {
    logTimeAndLevel(l);
    logPS.print(msg);
    logPS.print(" : ");
    logPS.println(Integer.toHexString(i));
  }

  // log the message and byte (regardless of level)
  private static void log(int l, String msg, byte b) {
    log(l, msg, (b & 0xff));
  }

  // log the message and short (regardless of level)
  private static void log(int l, String msg, short s) {
    log(l, msg, (s & 0xffff));
  }

  // log the message and long (regardless of level)
  private static void log(int l, String msg, long lng) {
    logTimeAndLevel(l);
    logPS.print(msg);
    logPS.print(" : ");
    logPS.println(Long.toHexString(lng));
  }

  // log the message and boolean (regardless of level)
  private static void log(int l, String msg, boolean b) {
    logTimeAndLevel(l);
    logPS.print(msg);
    logPS.print(" : ");
    logPS.println(b);
  }

  // log the message, with string and int (regardless of level)
  private static void log(int l, String msg, String msg2, int i) {
    logTimeAndLevel(l);
    logPS.print(msg);
    logPS.print(" : ");
    logPS.print(msg2);
    logPS.print(" : ");
    logPS.println(Integer.toHexString(i));
  }

  // log the message and two ints (regardless of level)
  private static void log(int l, String msg, int i1, int i2) {
    logTimeAndLevel(l);
    logPS.print(msg);
    logPS.print(" : ");
    logPS.print(Integer.toHexString(i1));
    logPS.print(" ");
    logPS.println(Integer.toHexString(i2));
  }

  // log the message and two longs (regardless of level)
  private static void log(int l, String msg, long l1, long l2) {
    logTimeAndLevel(l);
    logPS.print(msg);
    logPS.print(" : ");
    logPS.println(Long.toHexString(l1));
    logPS.print(" ");
    logPS.println(Long.toHexString(l2));
  }



  // common method to log start of line
  private static void logTimeAndLevel(int l) {
    // print out timestamp
    //logPS.print(sdf.format(new java.util.Date()));
    logPS.print(System.currentTimeMillis()); 
    
    // print out the level
    if (l == SEVERE) {
      logPS.print(" SEVERE: ");
    }
    else if (l == INFO) {
      logPS.print(" INFO: ");
    }
    else if (l == FINE) {
      logPS.print(" FINE: ");
    }
    else {
      logPS.print(" UNKNOWN: ");
    }
  }


  // Return our CVS id
  public static String getCVSID() {
    return "$Id: Log.java,v 1.7 2003/01/21 15:46:42 Bennett Stephen Exp $";
  }



}


/*
 * $Log: Log.java,v $
 * Revision 1.7  2003/01/21 15:46:42  Bennett Stephen
 * don't print performance info on error exit
 *
 * Revision 1.6  2002/11/28 19:32:29  Bennett Stephen
 * release candidate 1 version
 *
 * Revision 1.5  2002/10/12 14:23:46  Bennett Stephen
 * slight performance improvement
 *
 * Revision 1.4  2002/10/06 15:26:44  Bennett Stephen
 * moved away from jdk 1.4 logging
 *
 * Revision 1.3  2002/09/08 19:40:06  Bennett Stephen
 * extra Log functions added
 *
 * Revision 1.2  2002/08/10 08:22:24  Bennett Stephen
 * slightly more efficient logging
 *
 * Revision 1.1  2002/08/07 19:22:20  Bennett Stephen
 * Initial Version
 *
 *
 */

// ----------------------------------------------------
// Emacs magic - please do not remove the following lines
/*
 * Local Variables:
 * write-contents-hooks: ((lambda () (untabify (point-min) (point-max))))
 * End:
*/
// ----------------------------------------------------
